# coding: utf-8

"""
    Pure1 Public REST API

    Pure1 Public REST API, developed by [Pure Storage, Inc.](https://www.purestorage.com)   The Pure1 REST API 2.0 offers one single form of authentication: OAuth 2.0 via the [Token Exchange protocol](https://datatracker.ietf.org/doc/draft-ietf-oauth-token-exchange).  OAuth 2.0 is an open protocol to allow secure authorization in a simple and standard method from web, mobile, desktop and background applications.  Note that the [Authentication](#section/Authentication) section below mentions 'API Key' as the security scheme type. This is solely for the purpose of allowing testing this API with [Swagger UI](https://static.pure1.purestorage.com/api-swagger/index.html).  [Knowledge base reference documentation](https://support.purestorage.com/Pure1/Pure1_Manage/Pure1_Manage_-_REST_API/Pure1_Manage_-_REST_API__Reference)

    The version of the OpenAPI document: 1.2
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
from typing import Set, Dict, Any

from typing import List, Optional, Union
from pydantic import BaseModel, Field, StrictFloat, StrictInt, StrictStr, conlist
from pypureclient.pure1.Pure1_1_2.models.fixed_reference import FixedReference
from pypureclient.pure1.Pure1_1_2.models.invoice_line import InvoiceLine


class Invoice(BaseModel):
    """
    Invoice
    """
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total invoice amount, expressed in the currency defined in `currency`.")
    currency: Optional[StrictStr] = Field(default=None, description="The currency of the invoice in ISO 4217 format. This currency applies to the total invoice amount as well as all amounts in line items.")
    var_date: Optional[StrictInt] = Field(default=None, alias="date", description="The issuance date. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    due_date: Optional[StrictInt] = Field(default=None, description="The payment due date. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    end_user_name: Optional[StrictStr] = Field(default=None, description="The end user customer name.")
    end_user_purchase_order: Optional[StrictStr] = Field(default=None, description="The end user customer purchase order number.")
    id: Optional[StrictStr] = Field(default=None, description="The invoice number, a globally unique identifier for this invoice.")
    lines: Optional[conlist(InvoiceLine)] = Field(default=None, description="A list of invoice line items.")
    partner_purchase_order: Optional[StrictStr] = Field(default=None, description="The partner purchase order number.")
    payment_terms: Optional[StrictStr] = Field(default=None, description="The invoice payment terms.")
    sales_representative: Optional[StrictStr] = Field(default=None, description="The sales representative that issued the invoice.")
    ship_date: Optional[StrictInt] = Field(default=None, description="The invoice shipment date. Represented as a timestamp of 00:00 on that date in UTC, in milliseconds since UNIX epoch.")
    status: Optional[StrictStr] = Field(default=None, description="The invoice status. Values include `open` and `paid`.")
    subscription: Optional[FixedReference] = Field(default=None, description="A reference to which subscription this invoice belongs.")
    __properties = ["amount", "currency", "date", "due_date", "end_user_name", "end_user_purchase_order", "id", "lines", "partner_purchase_order", "payment_terms", "sales_representative", "ship_date", "status", "subscription"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.to_dict(include_readonly=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.as_request_dict())

    def as_request_dict(self) -> Dict[str, Any]:
        return self.to_dict(include_readonly=False)

    def to_dict(self, include_readonly: bool=True) -> Dict[str, Any]:

        """Returns the dictionary representation of the model using alias"""
        excluded_fields: Set[str] = set()
        if not include_readonly:
            excluded_fields.update([
            ])
        none_fields: Set[str] = set()
        for _field in self.__fields__.keys():
            if super().__getattribute__(_field) is None:
                none_fields.add(_field)

        _dict = self.dict(by_alias=True, exclude=excluded_fields, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in lines (list)
        if _include_in_dict('lines', include_readonly, excluded_fields, none_fields):
            _items = []
            for _item in self.lines:
                if _item:
                    _items.append(_item.to_dict(include_readonly=include_readonly))
            _dict['lines'] = _items
        # override the default output from pydantic by calling `to_dict()` of subscription
        if _include_in_dict('subscription', include_readonly, excluded_fields, none_fields):
            _dict['subscription'] = self.subscription.to_dict(include_readonly=include_readonly)
        return _dict

    def __getitem__(self, key):
        return super().__getattribute__(key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __delitem__(self, key):
        setattr(self, key, None)

    def __getattribute__(self, name: str) -> Any:
        _value = super().__getattribute__(name)
        if _value is None and name in self.__fields__.keys() and _should_raise_on_none():
            raise AttributeError
        return _value

    def __str__(self) -> str:
        return self.to_str()

    def __repr__(self) -> str:
        return self.to_str()

    @classmethod
    def from_json(cls, json_str: str) -> Invoice:
        """Create an instance of Invoice from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    @classmethod
    def from_dict(cls, obj: dict) -> Invoice:
        """Create an instance of Invoice from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return Invoice.parse_obj(obj)

        _obj = Invoice.construct(_fields_set=None, **{
            "amount": obj.get("amount"),
            "currency": obj.get("currency"),
            "var_date": obj.get("date"),
            "due_date": obj.get("due_date"),
            "end_user_name": obj.get("end_user_name"),
            "end_user_purchase_order": obj.get("end_user_purchase_order"),
            "id": obj.get("id"),
            "lines": [InvoiceLine.from_dict(_item) for _item in obj.get("lines")] if obj.get("lines") is not None else None,
            "partner_purchase_order": obj.get("partner_purchase_order"),
            "payment_terms": obj.get("payment_terms"),
            "sales_representative": obj.get("sales_representative"),
            "ship_date": obj.get("ship_date"),
            "status": obj.get("status"),
            "subscription": FixedReference.from_dict(obj.get("subscription")) if obj.get("subscription") is not None else None
        })
        return _obj

def _should_raise_on_none() -> bool:
    import importlib
    _package = importlib.import_module(__package__)
    return _package._attribute_error_on_none

def _include_in_dict(name: str, include_readonly: bool, excluded_fields: Set[str], none_fields: Set[str]) -> bool:
    if name in none_fields:
        return False
    return (include_readonly or name not in excluded_fields)

