# coding: utf-8

"""
    Pure1 Public REST API

    Pure1 Public REST API, developed by [Pure Storage, Inc.](https://www.purestorage.com)   The Pure1 REST API 2.0 offers one single form of authentication: OAuth 2.0 via the [Token Exchange protocol](https://datatracker.ietf.org/doc/draft-ietf-oauth-token-exchange).  OAuth 2.0 is an open protocol to allow secure authorization in a simple and standard method from web, mobile, desktop and background applications.  Note that the [Authentication](#section/Authentication) section below mentions 'API Key' as the security scheme type. This is solely for the purpose of allowing testing this API with [Swagger UI](https://static.pure1.purestorage.com/api-swagger/index.html).  [Knowledge base reference documentation](https://support.purestorage.com/Pure1/Pure1_Manage/Pure1_Manage_-_REST_API/Pure1_Manage_-_REST_API__Reference)

    The version of the OpenAPI document: 1.4
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json
from typing import Set, Dict, Any

from typing import Optional
from pydantic import BaseModel, Field, StrictInt, StrictStr
from pypureclient.pure1.Pure1_1_4.models.install_address import InstallAddress
from pypureclient.pure1.Pure1_1_4.models.sustainability_assessment import SustainabilityAssessment


class SustainabilityArray(BaseModel):
    """
    A single FlashArray/FlashBlade appliance Sustainability assessment information.
    """
    as_of: Optional[StrictInt] = Field(default=None, alias="_as_of", description="The freshness of the data (timestamp in millis since epoch).")
    id: Optional[StrictStr] = Field(default=None, description="A non-modifiable, globally unique ID chosen by the system.")
    name: Optional[StrictStr] = Field(default=None, description="A non-modifiable, locally unique name chosen by the system.")
    assessment: Optional[SustainabilityAssessment] = None
    install_address: Optional[InstallAddress] = Field(default=None, description="The address where the array is installed. This address is also where replacement parts will be shipped to.")
    reporting_status: Optional[StrictStr] = Field(default=None, description="Enum value that describes what is the status of the latest assessment. Valid values include: not_enough_data - There was not enough data   to calculate assessment level of the appliance. unsupported_purity_version - The appliance is running   an unsupported version of Purity operating system.   It is an old Purity version which does not provide   enough data for assessing the sustainability metrics. not_phoning_home - The appliance has not phoned home   for more than 7 days so the assessment level was not calculated assessment_ready - Assessment is ready and it is available under `assessment` field.")
    __properties = ["_as_of", "id", "name", "assessment", "install_address", "reporting_status"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.to_dict(include_readonly=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.as_request_dict())

    def as_request_dict(self) -> Dict[str, Any]:
        return self.to_dict(include_readonly=False)

    def to_dict(self, include_readonly: bool=True) -> Dict[str, Any]:

        """Returns the dictionary representation of the model using alias"""
        excluded_fields: Set[str] = set()
        if not include_readonly:
            excluded_fields.update([
                "as_of",
                "id",
                "name",
                "reporting_status",
            ])
        none_fields: Set[str] = set()
        for _field in self.__fields__.keys():
            if super().__getattribute__(_field) is None:
                none_fields.add(_field)

        _dict = self.dict(by_alias=True, exclude=excluded_fields, exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of assessment
        if _include_in_dict('assessment', include_readonly, excluded_fields, none_fields):
            _dict['assessment'] = self.assessment.to_dict(include_readonly=include_readonly)
        # override the default output from pydantic by calling `to_dict()` of install_address
        if _include_in_dict('install_address', include_readonly, excluded_fields, none_fields):
            _dict['install_address'] = self.install_address.to_dict(include_readonly=include_readonly)
        return _dict

    def __getitem__(self, key):
        return super().__getattribute__(key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def __delitem__(self, key):
        setattr(self, key, None)

    def __getattribute__(self, name: str) -> Any:
        _value = super().__getattribute__(name)
        if _value is None and name in self.__fields__.keys() and _should_raise_on_none():
            raise AttributeError
        return _value

    def __str__(self) -> str:
        return self.to_str()

    def __repr__(self) -> str:
        return self.to_str()

    @classmethod
    def from_json(cls, json_str: str) -> SustainabilityArray:
        """Create an instance of SustainabilityArray from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    @classmethod
    def from_dict(cls, obj: dict) -> SustainabilityArray:
        """Create an instance of SustainabilityArray from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SustainabilityArray.parse_obj(obj)

        _obj = SustainabilityArray.construct(_fields_set=None, **{
            "as_of": obj.get("_as_of"),
            "id": obj.get("id"),
            "name": obj.get("name"),
            "assessment": SustainabilityAssessment.from_dict(obj.get("assessment")) if obj.get("assessment") is not None else None,
            "install_address": InstallAddress.from_dict(obj.get("install_address")) if obj.get("install_address") is not None else None,
            "reporting_status": obj.get("reporting_status")
        })
        return _obj

def _should_raise_on_none() -> bool:
    import importlib
    _package = importlib.import_module(__package__)
    return _package._attribute_error_on_none

def _include_in_dict(name: str, include_readonly: bool, excluded_fields: Set[str], none_fields: Set[str]) -> bool:
    if name in none_fields:
        return False
    return (include_readonly or name not in excluded_fields)

